﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace HeuristicLab.Services.Authentication.TestClient2
{
    public partial class UserManagement : Form
    {

        private AuthenticationService auth;
        private DataTransfer.Application currentApplication;

        private DataTransfer.Application selectedApplication;
        private DataTransfer.User selectedUser;
        private DataTransfer.Role selectedRole;

        public UserManagement()
        {
            InitializeComponent();
            Initialize();
        }

        private void Initialize()
        {
            auth = new AuthenticationService();
            bsApplications.DataSource = auth.GetApplications();
            bsApplicationsCombo.DataSource = auth.GetApplications();
            RefreshLists();
        }

        private void RefreshLists()
        {
            if (currentApplication != null)
            {
                bsRoles.DataSource = auth.GetRoles(currentApplication.Id);
                bsUsers.DataSource = auth.GetUsers(currentApplication.Id);
            }
        }

        private void cbxApplications_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.currentApplication = (HeuristicLab.Services.Authentication.DataTransfer.Application)cbxApplications.SelectedItem;
            RefreshLists();
        }

        private void btnNewApplication_Click(object sender, EventArgs e)
        {
            // new
           //DataTransfer.Application  = new DataTransfer.Application();
            selectedApplication = (DataTransfer.Application)this.bsApplications.AddNew();
            
        }

        private void btnDeleteApplication_Click(object sender, EventArgs e)
        {
            if (this.dgvApplications.SelectedRows.Count > 0)
            {
                selectedApplication= ( DataTransfer.Application)this.dgvApplications.SelectedRows[0].DataBoundItem;
                if (selectedApplication != null)
                {
                    auth.DeleteApplication(selectedApplication.Id);
                    RefreshLists();
                }
            }
        }

        private void btnUpdateApplication_Click(object sender, EventArgs e)
        {
           
                if (selectedApplication != null)
                {
                    if (selectedApplication.Id.Equals(Guid.Empty))
                    {
                        // insert
                        selectedApplication.Id = Guid.NewGuid();
                        auth.InsertApplication(selectedApplication);
                    }
                    else
                    {
                        // update
                        auth.UpdateApplication(selectedApplication);
                    }
                }
            
            
        }

        private void dgvApplications_SelectionChanged(object sender, EventArgs e)
        {
            if (dgvApplications.SelectedRows.Count > 0)
            {
                selectedApplication = (DataTransfer.Application)dgvApplications.SelectedRows[0].DataBoundItem;
            }
        }

        private void dgvUsers_SelectionChanged(object sender, EventArgs e)
        {
            if (dgvUsers.SelectedRows.Count > 0)
            {
                selectedUser = (DataTransfer.User)dgvUsers.SelectedRows[0].DataBoundItem;
                RefreshUserRolesList();
            }
        }

        private void dgvRoles_SelectionChanged(object sender, EventArgs e)
        {
            if (dgvRoles.SelectedRows.Count > 0)
            {
                selectedRole = (DataTransfer.Role)dgvRoles.SelectedRows[0].DataBoundItem;
                RefreshRoleUsersList();
            }
        }

        private void btnUpdateRole_Click(object sender, EventArgs e)
        {
            if (selectedRole != null)
            {
                if (selectedRole.Id.Equals(Guid.Empty))
                {
                    // insert
                    selectedRole.Id = Guid.NewGuid();
                    selectedRole.ApplicationId = currentApplication.Id;

                    auth.InsertRole(selectedRole);
                }
                else
                {
                    // update
                    auth.UpdateRole(selectedRole);
                }
            }
        }

        private void btnNewRole_Click(object sender, EventArgs e)
        {
            selectedRole = (DataTransfer.Role)this.bsRoles.AddNew();
        }

        private void btnDeleteRole_Click(object sender, EventArgs e)
        {
            if (this.dgvRoles.SelectedRows.Count > 0)
            {
                selectedRole = (DataTransfer.Role)this.dgvRoles.SelectedRows[0].DataBoundItem;
                if (selectedRole != null)
                {
                    auth.DeleteRole(selectedRole.Id);
                    RefreshLists();
                }
            }
        }

        private void btnNewUser_Click(object sender, EventArgs e)
        {
            selectedUser = (DataTransfer.User)this.bsUsers.AddNew();
        }

        private void btnDeleteUser_Click(object sender, EventArgs e)
        {
            if (this.dgvUsers.SelectedRows.Count > 0)
            {
                selectedUser = (DataTransfer.User)this.dgvUsers.SelectedRows[0].DataBoundItem;
                if (selectedUser != null)
                {
                    auth.DeleteUser(selectedUser.Id);
                    RefreshLists();
                }
            }
        }

        private void btnUpdateUser_Click(object sender, EventArgs e)
        {
            if (selectedUser != null)
            {
                if (selectedUser.Id.Equals(Guid.Empty))
                {
                    // insert
                    selectedUser.Id = Guid.NewGuid();
                    selectedUser.ApplicationId = currentApplication.Id;

                    auth.InsertUser(selectedUser);
                }
                else
                {
                    // update
                    auth.UpdateUser(selectedUser);
                }
            }
        }

        private void btnRemoveRoleFromUser_Click(object sender, EventArgs e)
        {
            if (this.dgvUserRoles.SelectedRows.Count > 0)
            {

                DataTransfer.Role role = (DataTransfer.Role)this.dgvUserRoles.SelectedRows[0].DataBoundItem;
                if (role != null)
                {
                    auth.RemoveUserFromRole(role.Id, selectedUser.Id);
                    RefreshUserRolesList();
                }
            }
        }

        private void btnAddRoleToUser_Click(object sender, EventArgs e)
        {
            if (this.dgvUserRolesAll.SelectedRows.Count > 0)
            {
                DataTransfer.Role role = (DataTransfer.Role)this.dgvUserRolesAll.SelectedRows[0].DataBoundItem;
                if (role != null)
                {
                    auth.AddUserToRole( role.Id,selectedUser.Id);
                    RefreshUserRolesList();
                }
            }
        }

        private void RefreshUserRolesList()
        {
            bsUserRoles.DataSource = auth.GetRolesForUser(selectedUser.Id);
        }

        private void RefreshRoleUsersList()
        {
            bsRoleUsers.DataSource = auth.GetUsersInRole(selectedRole.Id);
        }

        private void btnRemoveUserFromRole_Click(object sender, EventArgs e)
        {
            if (this.dgvRoleUsers.SelectedRows.Count > 0)
            {

                DataTransfer.User user = (DataTransfer.User)this.dgvRoleUsers.SelectedRows[0].DataBoundItem;
                if (user != null)
                {
                    auth.RemoveUserFromRole(selectedRole.Id, user.Id);
                    RefreshRoleUsersList();
                }
            }
        }

        private void btnAddUserToRole_Click(object sender, EventArgs e)
        {
            if (this.dgvRoleUsersAll.SelectedRows.Count > 0)
            {
                DataTransfer.User user = (DataTransfer.User)this.dgvRoleUsersAll.SelectedRows[0].DataBoundItem;
                if (user != null)
                {
                    auth.AddUserToRole(selectedRole.Id, user.Id);
                    RefreshRoleUsersList();
                }
            }
        }

  
    }
}
